//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
// 

#include "Country.h"

Define_Module(Country);

void Country::initialize()
{
    simtime_t upheaval = par("upheavalTime");
    int cols = this->getParentModule()->par("columns");
    int rows = this->getParentModule()->par("rows");
    float noOfCountries = cols * rows;
    this->inUpheaval = false;
    this->overlord = getIndex();
    WATCH(overlord);
    this->overlordColor = hsvToRgb(this->overlord/noOfCountries,uniform(0.4,1),uniform(0.4,1));
    upheavalEvent = new cMessage("upheaval");
    scheduleAt(upheaval, upheavalEvent);
}

Country::~Country()
{
    cancelAndDelete(upheavalEvent);
}

void Country::handleMessage(cMessage *msg)
{
    // TODO - Generated method body
    if (msg==upheavalEvent)
    {
        if (!this->inUpheaval)
        {
            UNmsg *un = new UNmsg("New Empire Emerges");
            un->setNewGovernment(this->overlord);
            cModule *targetModule = getParentModule()->getSubmodule("UN");
            sendDirect(un, targetModule, "in");
        }
        this->inUpheaval = true;
        this->getDisplayString().setTagArg("b",3, this->overlordColor.c_str());
        std::vector<const char *> gateVector = this->getGateNames();
        for (std::vector<const char *>::iterator it = gateVector.begin() ; it != gateVector.end(); ++it)
        {
            char gateName[20];
            sprintf(gateName, "%s$o", *it);
            if (this->gate(gateName)->isConnected())
            {
                Warmsg *war = new Warmsg("war");
                war->setArmyColor(this->overlordColor.c_str());
                war->setArmyIndex(this->overlord);
                send(war, gateName);
            }
        }
        this->getDisplayString().setTagArg("i", 0, "block/circle");
    }
    else if (strcmp(msg->getName(),"war") == 0)
    {
        Warmsg *war = check_and_cast<Warmsg *>(msg);
        if (war->getArmyIndex() != this->overlord)
        {
            double chanceOfDefeat = par("chanceOfDefeat");
            double chanceOfDefeatUpheaval = par("chanceOfDefeatUpheaval");
            double retaliationChance = par("retaliationChance");
            double odds = this->inUpheaval ? chanceOfDefeatUpheaval : chanceOfDefeat;
            if (uniform(0,1) < odds)
            {
                this->getDisplayString().setTagArg("b",3,war->getArmyColor());
                this->overlordColor = war->getArmyColor();
                int oldOverlord = this->overlord;
                this->overlord = war->getArmyIndex();
                this->inUpheaval = true;
                cModule *targetModule = getParentModule()->getSubmodule("UN");
                UNmsg *un = new UNmsg("Change of government");
                un->setOldGovernment(oldOverlord);
                un->setNewGovernment(this->overlord);
                sendDirect(un, targetModule, "in");
                cancelEvent(upheavalEvent);
                double campaignContinueChance = par("campaignContinueChance");
                this->getDisplayString().setTagArg("i", 0, "block/circle");
                if (uniform(0,1) < campaignContinueChance)
                {
                    this->getDisplayString().setTagArg("i", 0, "block/broadcast");
                    double campaignTime = par("campaignTime");
                    scheduleAt(simTime()+campaignTime, upheavalEvent);
                }
            }
            else if (this->inUpheaval && uniform(0,1) < retaliationChance)
            {
                cancelEvent(upheavalEvent);
                this->getDisplayString().setTagArg("i", 0, "block/wrx");
                double retaliationTime = par("retaliationTime");
                scheduleAt(simTime()+retaliationTime, upheavalEvent);
            }
        }
        delete war;
    }
}

std::string hsvToRgb(float h, float s, float v)
{
    float r, g, b;

    int i = h * 6;
    float f = h * 6 - i;
    float p = v * (1 - s);
    float q = v * (1 - f * s);
    float t = v * (1 - (1 - f) * s);

    switch(i % 6){
        case 0: r = v, g = t, b = p; break;
        case 1: r = q, g = v, b = p; break;
        case 2: r = p, g = v, b = t; break;
        case 3: r = p, g = q, b = v; break;
        case 4: r = t, g = p, b = v; break;
        case 5: r = v, g = p, b = q; break;
    }

    char color[20];
    sprintf(color, "#%02x%02x%02x", int(r * 255), int(g * 255), int(b * 255));
    return std::string(color);
}
